
# q-expansion of newform 64.12.a.j, downloaded from the LMFDB on 14 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 64
weight = 12
poly_data = [-3, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 240], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [0, 1], [10, 0], [0, 62], [0, -1957], [148722, 0], [-1904606, 0], [0, -1531], [0, -126278], [118019810, 0], [0, 237144], [390850634, 0], [-992147110, 0], [0, 3108275], [0, 1171636], [-3538213222, 0], [0, -14302777], [-11957919710, 0], [0, 24084641], [0, -55312498], [-7519762038, 0], [0, 70883724], [0, 85401605], [-59057953990, 0], [-76766507694, 0], [66172877002, 0], [0, -519659066], [0, -116196941], [93919574706, 0], [135852019538, 0], [0, 163593472], [0, 1039412049], [704409018106, 0], [0, 2062011947], [115382868314, 0], [0, 2710491418], [1225233017666, 0], [0, -2961944471], [0, 3176261334], [475014247378, 0], [0, -899360803], [-1005004560582, 0], [0, 7845310832], [-3763283674382, 0], [-7227640688246, 0], [0, 18765683662], [0, -4154392507], [0, -20647018648], [0, 8924059993], [3304810595370, 0], [10056865064938, 0], [0, 23149747364], [-17010519831870, 0], [0, -47146170825], [-6769365257726, 0], [0, -37049381090], [17617056413074, 0], [0, 91678709948], [12436331914746, 0], [40192529160890, 0], [0, 134659853743], [-63888907462678, 0], [0, -159577051283], [0, 46230273810], [-14727265544182, 0], [-46700913187454, 0], [0, -45292877045], [-153250230976974, 0], [0, 37249701391], [123054584583010, 0], [-112950271633438, 0], [0, 436137075462], [0, 311684550692], [140379399587098, 0], [0, -318186501321], [0, -566327346528], [-116649059042134, 0], [121030804407634, 0], [155669182157602, 0], [214248722811306, 0], [0, -391634519399], [159992267849130, 0], [0, 1071716656436], [224719740682882, 0], [0, -671234909838], [0, -1304811175657], [4382131817714, 0], [-203549629608006, 0], [-87297577915438, 0], [0, 1226095230508], [0, 1360037386133], [0, -1767547417432], [0, -1975639110458], [0, 1114653132227], [0, -2172799362355], [0, 1542477010178], [-1440076208733790, 0], [-1258692271202822, 0], [0, 102668585787], [-1521859903890558, 0], [0, 372442740921], [-684793188993646, 0], [0, 714524465965], [-633864432999926, 0], [0, 4955316247031], [1965341994026114, 0], [0, -4198188169941], [-2016379738535438, 0], [0, -1649974460726], [752984859301690, 0], [0, -2399432345752], [2315690440413802, 0], [-2300576075306326, 0], [0, 54263250771], [0, 2790767872098], [-2929577783651150, 0], [0, 103202032497], [0, -8135442345026], [1186005309504658, 0], [0, -246578068459], [-825683356642150, 0], [-20198065783918, 0], [-6025694142165686, 0], [0, -1513395358925], [0, 11438397785821], [-4975991470540510, 0], [-7742962224708630, 0], [0, -16225462400724], [0, -5491104637910], [5821786414809378, 0], [0, -11290277926567], [0, -17686036672634], [0, 6140633721828], [4964608124256186, 0], [-4377531662291638, 0], [-2086105780982350, 0], [936216003555242, 0], [0, 12946804361685], [-2823146578119614, 0], [15821587343026010, 0], [0, 6242197747971], [-3783171304768550, 0], [0, -24397116783342], [0, 2471972180567], [-7333081102646046, 0], [0, 21175289449326], [22126926582118202, 0], [8139318404243306, 0], [0, -42733687076145], [0, -17084629150840], [17954893369351666, 0], [-7816644489354350, 0], [0, 13766959984221], [0, -27820805562174], [0, -48162864761461], [0, 26189133443420], [0, -44875189475942], [14008114272185746, 0], [-39175088855913686, 0], [16283422261689458, 0], [0, 69908948997453], [-20610116635383542, 0], [0, 32428214725966], [0, -101880887211365], [7564548890673506, 0], [0, 38201237770922], [0, -136039093602488], [-18475642343324534, 0], [51256511630872770, 0], [48744226512888442, 0], [0, -21735755678169], [-49594827556833022, 0], [0, -102476138879330], [-5621103701425398, 0], [0, 156430427036604], [13401584461543610, 0], [0, 71478738158559], [54705558608028938, 0], [0, 93126794153014], [-8620635421159726, 0], [0, -193166456464912], [0, 214177782921169], [16244977833618378, 0], [-15317119942136374, 0], [0, 24499520802668], [87776657365351450, 0], [-56524855256626686, 0], [0, -147815247675303], [37117929593081754, 0], [0, -50175938593696], [33942639221398338, 0], [0, -190432026052821], [0, -152308167473851], [-106409118140308510, 0], [0, -281603217860823], [36562437191285098, 0], [-42378743093575150, 0], [-157198116541811742, 0], [-44627639918659774, 0], [0, -137186231912466], [108705005243329330, 0], [0, 136806461606444], [-138669463850083974, 0], [-76490638505659230, 0], [0, -14773338682861], [32134534728852226, 0], [0, 466224283792320], [0, 113328052211809], [46201766698365130, 0], [0, -319374753933285], [-66699235404654046, 0], [143735433981194650, 0], [0, -50421267677542], [0, -230943558970353], [0, -269398287564042], [4246081820896090, 0], [0, -673343668635436], [38196756858455138, 0], [0, 771708082699530], [-86017278110704478, 0], [-232111451084858070, 0], [0, 454173362040034], [126758374733916850, 0], [0, -319083915571396], [0, -190246716948891], [12360181556821946, 0], [398926093562349562, 0], [0, -355492700181544], [0, -687890362512778], [0, 483979244767571], [324158053727832882, 0], [0, 773444415025197], [0, 686134463661840], [-313368090608456630, 0], [0, 484658162053019], [0, -404770205183316], [-418763515463732766, 0], [-61320445384761382, 0], [0, -1050148994030881], [0, 799665616574726], [0, -1258178534539251], [0, 1227623244705863], [0, 1023160812108830], [223427615850718130, 0], [2133141112569202, 0], [0, -598817228424934], [0, 456614788226648], [0, 1255815587269721], [0, 1052812939362067], [0, 139819408165652], [-272809629323448126, 0], [135638365304528002, 0], [0, 178829359553870], [-549015514269579894, 0], [-584226934705164782, 0], [0, -160604355807771], [26307292914676122, 0], [0, -1733673079436561], [439164451181627786, 0], [314990783981127754, 0], [0, -710131312500000], [0, 901543875230753], [721156223482168170, 0], [839118454969485602, 0], [-69227193640781806, 0], [0, -1358239967194599], [-235329308204998670, 0], [-1129634998002557222, 0], [0, 932615610601063], [1060343515167645482, 0], [-769965995477886542, 0], [0, -302586154720811], [568134428154491898, 0], [0, -613353019110328], [-584189006750022126, 0], [0, 3272852462616322], [0, 567282951756423], [712367403535176610, 0], [559173576429012170, 0], [0, 1183544656169669], [0, -3613025384956712], [0, -638926115629482], [0, -1456079290954574], [-769143331096827062, 0], [0, -1051420640472549], [0, -2667842840019668], [699058957581612258, 0], [-1375266450104971846, 0], [0, -2165997014480982], [-1611551771880008734, 0], [-1605561281556825230, 0], [0, -4423258412087299], [777084598742631962, 0], [0, -3939514011237189], [-522585072934199662, 0], [1540362835130179586, 0], [0, 1030124180844792], [1430218385097369082, 0], [0, 1193789124388535], [0, 4660010064904097], [1417585583590957882, 0], [175599181612872626, 0], [0, 3534890713892364], [0, 3208710536060389], [0, -1657257185033953], [-932089061822933934, 0], [0, 1443931924105843], [780989056841561490, 0], [0, -4621778420121502], [427552099426933162, 0], [0, 1877463793011580], [1403760054226020250, 0], [2061278295724446482, 0], [0, 3962782863213721], [0, -149186010283978], [1410109201413867690, 0], [0, 2470987729246557], [0, 3606536788588976], [-1571380115958365502, 0], [2283182173268866546, 0], [0, -2099935323726075], [938968207013187946, 0], [1055469983665781090, 0], [0, -6494070612485784], [1410581968352546842, 0], [-2364701439938795070, 0], [0, 9180009776285153], [0, 8841580072482351], [0, 7716598337470328], [4248498746463977578, 0], [-755349616125220590, 0], [2007339883506303106, 0], [0, -196912607132912], [0, -4540115354966911], [0, 2429044896330779], [0, 3779918764334607], [-83653411767214174, 0], [567767716963197538, 0], [-5475686981664555990, 0], [0, -7109997023505980], [2723035023881763962, 0], [439859502886652234, 0], [2140442444548125994, 0], [0, -8193173499407650], [-149240099976495358, 0], [0, 8258410354860297], [2938246265883458730, 0], [0, -314901866202109], [0, -12850618026700204], [3204583797819547514, 0], [0, -16916346406626959], [1082702602039515834, 0], [-3410019326768558030, 0], [0, 6541557869918860], [-463284986471410150, 0], [-3368165939588657798, 0], [0, 14756465635837480], [0, -516211410993149], [7168499655865968706, 0], [0, 1917872425879874], [-30213063892979734, 0], [2090275797630365258, 0], [0, -7514718628113892], [0, -20382428847209761], [0, 10536228157224009], [-4004694263746512022, 0], [-5096494692049815406, 0], [0, 4787510818386926], [-4728314985977284630, 0], [0, 10162397201255273], [0, -19055464267741197], [0, 17163982608782532], [-890942794287002086, 0], [0, -12626899764811710], [-221830140214927806, 0], [0, 964977603899029], [0, -9747877716278056], [-4115650146556485278, 0], [-4913580399377406206, 0], [4786095223962816906, 0], [-8170956684500689022, 0], [-8758602479194735558, 0], [0, 17852521790463594], [9332928148769621026, 0], [0, 15903966495606521], [0, 23310540625873670], [0, 19676431471399780], [-3757545893608818726, 0], [0, -23345904208290153], [0, -3203281127818720], [2984388709022439234, 0], [-8287264060925302742, 0], [0, 16969496705392923], [0, -4475479399309803], [0, 25960411907181050], [13900345361043166122, 0], [0, -19843616128198888], [2850927286221816410, 0], [0, 12105888649073459], [9117461733801554650, 0], [-2637940847092532414, 0], [-5959672939682148622, 0], [0, 11354200658631340], [-7878159703885974934, 0], [9272511597324194666, 0], [0, -6705653463358042], [13919152858641236946, 0], [7449555130770111298, 0], [0, 5846753183347789], [0, 11794685763762801], [-13697741400416065678, 0], [3515279188620841274, 0], [0, 21273884854044959], [0, 1760908339056809], [2161855421544053226, 0], [15800452086825090962, 0], [0, 686428910719920], [0, 8355010883878606], [-9635906004661727134, 0], [0, -17530919217496950], [-20058300824100835006, 0], [-7284514174758506806, 0], [0, 34472212884328228], [0, -17754713594125385], [18241162257011799418, 0], [6291813763867827506, 0], [0, 30451940375690629], [-21380851659199904774, 0], [0, 51619724038410511], [0, 19015621554295410]]
